@@ -4,10 +4,12 @@ import numpy as np
 import copy
 import matplotlib.pyplot as plt
 import scipy.optimize as optim
-import warnings
+import logging
 
 from .ext import gp_c
 
+logger = logging.getLogger("gp.gp")
+
 DTYPE = np.float64
 EPS = np.finfo(DTYPE).eps
 MIN = np.log(np.exp2(DTYPE(np.finfo(DTYPE).minexp + 4)))
@@ -94,6 +96,8 @@ class GP(object):
             self._memoized = {}
             self._x = val.astype(DTYPE)
             self.n, = self.x.shape
+        else: # pragma: no cover
+            pass
 
     @property
     def y(self):
@@ -116,6 +120,8 @@ class GP(object):
             self._y = val.astype(DTYPE)
             if self.y.shape != (self.n,):
                 raise ValueError("invalid shape for y: %s" % str(self.y.shape))
+        else: # pragma: no cover
+            pass
 
     @property
     def s(self):
@@ -132,6 +138,9 @@ class GP(object):
 
     @s.setter
     def s(self, val):
+        if val < 0:
+            raise ValueError("invalid value for s: %s" % val)
+
         if val != self._s:
             self._memoized = {}
             self._s = DTYPE(val)
@@ -158,6 +167,8 @@ class GP(object):
             self._memoized = {}
             self.K.params = val[:-1]
             self.s = val[-1]
+        else: # pragma: no cover
+            pass
 
     def copy(self):
         """
@@ -303,7 +314,11 @@ class GP(object):
         """
         y = self._y
         K = self.Kxx
-        Kiy = self.inv_Kxx_y
+        try:
+            Kiy = self.inv_Kxx_y
+        except np.linalg.LinAlgError:
+            return -np.inf
+
         return DTYPE(gp_c.log_lh(y, K, Kiy))
 
     @memoprop
@@ -361,7 +376,12 @@ class GP(object):
 
         y = self._y
         dloglh = np.empty(len(self.params))
-        Ki = self.inv_Kxx
+        try:
+            Ki = self.inv_Kxx
+        except np.linalg.LinAlgError:
+            dloglh.fill(np.nan)
+            return dloglh
+
         Kj = self.Kxx_J
         Kiy = self.inv_Kxx_y
         gp_c.dloglh_dtheta(y, Ki, Kj, Kiy, self._s, dloglh)
@@ -388,7 +408,12 @@ class GP(object):
 
         y = self._y
         dlh = np.empty(len(self.params))
-        Ki = self.inv_Kxx
+        try:
+            Ki = self.inv_Kxx
+        except np.linalg.LinAlgError:
+            dlh.fill(np.nan)
+            return dlh
+
         Kj = self.Kxx_J
         Kiy = self.inv_Kxx_y
         lh = self.lh
@@ -415,13 +440,19 @@ class GP(object):
         """
 
         y = self._y
-        Ki = self.inv_Kxx
+        d2lh = np.empty((len(self.params), len(self.params)))
+        try:
+            Ki = self.inv_Kxx
+        except np.linalg.LinAlgError:
+            d2lh.fill(np.nan)
+            return d2lh
+
         Kj = self.Kxx_J
         Kh = self.Kxx_H
         Kiy = self.inv_Kxx_y
         lh = self.lh
         dlh = self.dlh_dtheta
-        d2lh = np.empty((len(self.params), len(self.params)))
+
         gp_c.d2lh_dtheta2(y, Ki, Kj, Kh, Kiy, self._s, lh, dlh, d2lh)
         return d2lh
 
@@ -623,8 +654,7 @@ class GP(object):
         ax.plot(x, y, 'o', ms=7, color=markercolor)
         ax.set_xlim(*xlim)
 
-    def fit_MLII(self, params_to_fit, bounds=None,
-                 randf=None, nrestart=5, verbose=False):
+    def fit_MLII(self, params_to_fit, randf=None, nrestart=5, verbose=False):
         """
         Fit parameters of the gaussian process using MLII (maximum
         likelihood) estimation.
@@ -642,12 +672,6 @@ class GP(object):
             parameters, indicating which parameters should be
             fit. Parameters which are not fit keep their current value.
 
-        bounds : list of tuples (optional)
-            The upper and lower bounds for each parameter that is being
-            fit; use ``None`` to specify no bound. If not specified,
-            the bounds default to ``(EPS, None)``, where
-            ``EPS=numpy.finfo(float).eps``.
-
         randf : list of functions (optional)
             A list of functions to give an initial starting value for
             each parameter that is being fit. The functions should
@@ -669,9 +693,6 @@ class GP(object):
         # boolean array of which parameters to fit
         fitmask = np.array(params_to_fit, dtype='bool')
 
-        # default for bounds
-        if bounds is None:
-            bounds = tuple((EPS, None) for p in params_to_fit if p)
         # default for randf
         if randf is None:
             randf = tuple(
@@ -690,44 +711,56 @@ class GP(object):
 
         # update the GP object with new parameter values
         def new_params(theta):
-            th = np.array(theta).ravel()
-            out = [params[i] if j is None else th[j]
+            out = [params[i] if j is None else theta[j]
                    for i, j in enumerate(iparam)]
             return out
 
         # negative log likelihood
         def f(theta):
-            self.params = new_params(theta)
-            out = -self.log_lh
+            params = new_params(theta)
+            try:
+                self.params = params
+            except ValueError:
+                out = np.inf
+            else:
+                out = -self.log_lh
             return out
 
         # jacobian of the negative log likelihood
         def df(theta):
-            self.params = new_params(theta)
-            out = -self.dloglh_dtheta[fitmask]
+            params = new_params(theta)
+            try:
+                self.params = params
+            except ValueError:
+                out = np.empty(len(theta))
+                out.fill(np.inf)
+            else:
+                out = -self.dloglh_dtheta[fitmask]
             return out
 
-        def cb(theta):
-            print theta
-
         # run the optimization a few times to find the best fit
-        args = np.empty((nrestart, len(bounds)))
+        args = np.empty((nrestart, len(randf)))
         fval = np.empty(nrestart)
         for i in xrange(nrestart):
-            p0 = tuple(r() for r in randf)
-            self.params = new_params(p0)
-            if verbose:
-                print "      p0 = %s" % (p0,)
+            p0_i = tuple(r() for r in randf)
+            self.params = new_params(p0_i)
+            logger.debug("p0 = %s", p0_i)
             popt = optim.minimize(
-                fun=f, x0=p0, jac=df, method='L-BFGS-B', bounds=bounds)
+                fun=f, x0=p0_i, jac=df, method='L-BFGS-B',
+                options={'disp': verbose})
+
             args[i] = popt['x']
             fval[i] = popt['fun']
-            if verbose:
-                print "      -MLL(%s) = %f" % (args[i], fval[i])
+
+            logger.debug("-MLL(%s) = %f", args[i], fval[i])
+            if not popt['success'] or np.isinf(fval[i]):
+                logger.warning("MLII failed, p=%s" % args[i])
+                fval[i] = np.nan
 
         # choose the parameters that give the best MLL
-        if args is None or fval is None:
+        if np.isnan(fval).all():
             raise RuntimeError("Could not find MLII parameter estimates")
 
         # update our parameters
-        self.params = new_params(args[np.argmin(fval)])
+        self.params = new_params(args[np.nanargmin(fval)])
+        logger.info("Best parameters: %s", self.params)
