@@ -42,7 +42,7 @@ class Kernel(object):
         """
         raise NotImplementedError
 
-    def K(self, x1, x2):
+    def K(self, x1, x2, out=None):
         r"""
         Kernel function evaluated at `x1` and `x2`.
 
@@ -62,10 +62,10 @@ class Kernel(object):
         raise NotImplementedError
 
     @wraps(K)
-    def __call__(self, x1, x2):
-        return self.K(x1, x2)
+    def __call__(self, x1, x2, out=None):
+        return self.K(x1, x2, out=out)
 
-    def jacobian(self, x1, x2):
+    def jacobian(self, x1, x2, out=None):
         r"""
         Jacobian of the kernel function evaluated at `x1` and `x2`.
 
@@ -85,7 +85,7 @@ class Kernel(object):
         """
         raise NotImplementedError
 
-    def hessian(self, x1, x2):
+    def hessian(self, x1, x2, out=None):
         r"""
         Hessian of the kernel function evaluated at `x1` and `x2`.
 
